@@ -33,6 +33,7 @@
 
 #include <KColorScheme>
 #include <KLocalizedString>
+#include <KMessageBox>
 #include <QDebug>
 
 struct AnimationData
@@ -76,7 +77,7 @@ QString WorksheetEntry::colorNames[] = {i18n("White"), i18n("Black"),
 WorksheetEntry::WorksheetEntry(Worksheet* worksheet) : QGraphicsObject(), m_controlElement(worksheet, this)
 {
     worksheet->addItem(this);
-
+    setAcceptHoverEvents(true);
     connect(&m_controlElement, &WorksheetControlItem::drag, this, &WorksheetEntry::startDrag);
 }
 
@@ -370,11 +371,15 @@ void WorksheetEntry::keyPressEvent(QKeyEvent* event)
     case Qt::Key_Up:
         if (event->modifiers() == Qt::NoModifier)
             moveToPreviousEntry(WorksheetTextItem::BottomRight, 0);
+        else if (event->modifiers() == Qt::CTRL)
+            moveToPrevious();
         break;
     case Qt::Key_Right:
     case Qt::Key_Down:
         if (event->modifiers() == Qt::NoModifier)
             moveToNextEntry(WorksheetTextItem::TopLeft, 0);
+        else if (event->modifiers() == Qt::CTRL)
+            moveToNext();
         break;
         /*case Qt::Key_Enter:
     case Qt::Key_Return:
@@ -382,10 +387,6 @@ void WorksheetEntry::keyPressEvent(QKeyEvent* event)
             evaluate();
         else if (event->modifiers() == Qt::ControlModifier)
             worksheet()->insertCommandEntry();
-        break;
-    case Qt::Key_Delete:
-        if (event->modifiers() == Qt::ShiftModifier)
-            startRemoving();
         break;*/
     default:
         event->ignore();
@@ -402,7 +403,10 @@ void WorksheetEntry::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 
 void WorksheetEntry::populateMenu(QMenu* menu, QPointF pos)
 {
-    auto* firstAction = menu->actions().first();
+    QAction* firstAction = nullptr;
+    if (!menu->actions().isEmpty()) //action() can be empty, s.a. WorksheetTextItem::populateMenu() where this function is called
+        firstAction = menu->actions().first();
+
     QAction* action;
     if (!worksheet()->isRunning() && wantToEvaluate())
     {
@@ -412,19 +416,26 @@ void WorksheetEntry::populateMenu(QMenu* menu, QPointF pos)
         menu->insertSeparator(firstAction);
     }
 
-    action = new QAction(QIcon::fromTheme(QLatin1String("go-up")), i18n("Move Up"));
-//     connect(action, &QAction::triggered, this, &WorksheetEntry::moveToPrevious); //TODO: doesn't work
-    connect(action, SIGNAL(triggered()), this, SLOT(moveToPrevious()));
-    menu->insertAction(firstAction, action);
+    if (m_prev) {
+        action = new QAction(QIcon::fromTheme(QLatin1String("go-up")), i18n("Move Up"));
+    //     connect(action, &QAction::triggered, this, &WorksheetEntry::moveToPrevious); //TODO: doesn't work
+        connect(action, SIGNAL(triggered()), this, SLOT(moveToPrevious()));
+        action->setShortcut(Qt::CTRL + Qt::Key_Up);
+        menu->insertAction(firstAction, action);
+    }
 
-    action = new QAction(QIcon::fromTheme(QLatin1String("go-down")), i18n("Move Down"));
-//     connect(action, &QAction::triggered, this, &WorksheetEntry::moveToNext); //TODO: doesn't work
-    connect(action, SIGNAL(triggered()), this, SLOT(moveToNext()));
-    menu->insertAction(firstAction, action);
-    menu->insertSeparator(firstAction);
+    if (m_next) {
+        action = new QAction(QIcon::fromTheme(QLatin1String("go-down")), i18n("Move Down"));
+    //     connect(action, &QAction::triggered, this, &WorksheetEntry::moveToNext); //TODO: doesn't work
+        connect(action, SIGNAL(triggered()), this, SLOT(moveToNext()));
+        action->setShortcut(Qt::CTRL + Qt::Key_Down);
+        menu->insertAction(firstAction, action);
+        menu->insertSeparator(firstAction);
+    }
 
     action = new QAction(QIcon::fromTheme(QLatin1String("edit-delete")), i18n("Remove"));
     connect(action, &QAction::triggered, this, &WorksheetEntry::startRemoving);
+    action->setShortcut(Qt::ShiftModifier + Qt::Key_Delete);
     menu->insertAction(firstAction, action);
     menu->insertSeparator(firstAction);
 
@@ -699,6 +710,10 @@ bool WorksheetEntry::aboutToBeRemoved()
 
 void WorksheetEntry::startRemoving()
 {
+    int rc = KMessageBox::warningYesNo(nullptr, i18n("Do you really want to remove this entry?"), i18n("Remove Entry"));
+    if (rc == KMessageBox::No)
+        return;
+
     if (!worksheet()->animationsEnabled()) {
         m_aboutToBeRemoved = true;
         remove();
@@ -785,6 +800,8 @@ void WorksheetEntry::remove()
 
     // make the entry invisible to QGraphicsScene's itemAt() function
     forceRemove();
+
+    worksheet()->setModified();
 }
 
 void WorksheetEntry::setSize(QSizeF size)
@@ -980,6 +997,8 @@ void WorksheetEntry::moveToNext(bool updateLayout)
 
         if (updateLayout)
             worksheet()->updateLayout();
+
+        worksheet()->setModified();
     }
 }
 
@@ -1010,6 +1029,8 @@ void WorksheetEntry::moveToPrevious(bool updateLayout)
 
         if (updateLayout)
             worksheet()->updateLayout();
+
+        worksheet()->setModified();
     }
 }
 
